#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 17 21:58:04 2020

@author: cmauger
"""

'''--- Importation des bibliothèque ----------------------------------------'''
from pyfirmata import Arduino, util  # bibliothèque qui renferme les commandes Python
                                     # compréhensibles par la carte Arduino
import time  # bibliothèque qui fournit différentes fonctions liées au temps
import numpy as np   # bibliothèque qui gère les outils mathématiques
import matplotlib.pyplot as plt  # bibliothèque qui permet de tracer les graphiques
'''-------------------------------------------------------------------------'''

'''--- Communication Arduino -----------------------------------------------'''
Carte = Arduino('/dev/ttyACM0')  # Attribuer un nom « Carte » à la carte et à son adresse
                                 # /dev/ttyACM0 ». Cette adresse doit être la même
                                 # que celle indiquée par le logiciel Arduino
Acqui = util.Iterator(Carte)  # lancement de l'acquisition
Acqui.start()  # idem : lancement de l'acquisition
led13 = Carte.get_pin('d:13:o')  # sortie digitale 13 (petite led)
InA0 = Carte.get_pin('a:0:i')  # entrée analogique A0
time.sleep(1.0)  # génère une pause de 1s indispensable pour l’initialisation de la carte
'''-------------------------------------------------------------------------'''

'''--- Données physiques ---------------------------------------------------'''
rho = 997  # masse volumique en [kg.m⁻3]
g = 9.81  # accélération gravitationnelle [m.s⁻2]
D = input('Quel est le diamètre du contenant ? en [m]  ')  # récupérer le diamètre du contenant
'''-------------------------------------------------------------------------'''

'''--- Paramètres script ---------------------------------------------------'''
kk = 5  # nombre de mesures (à définir selon l’expérience menée)
Data = np.zeros((kk, 5))  # matrice de stockage des mesures/résultats, n° de la mesure [-], 
                          # hauteur théorique, tension mesurée [V],  pression relative [Pa],
                          # hauteur de liquide [m]
UMes = []  # initialisation de la variable Umes (tension mesurée sur l’entrée A0)
VolCumul = 0  # volume initial de liquide dans le contenant [L]
'''-------------------------------------------------------------------------'''

'''--- Acquisiton et calcul----------------------------------------------------'''
for i in range(0, kk):  # définition d’une boucle « for » qui va répéter un bloc 
                        # d’instructions i fois avec i allant de 0 à kk   
	Vol = input('Quelle quantité d\'eau a été ajoutée ? en litre  ')  # récupère la quantité d’eau ajoutée
	VolCumul =  VolCumul + float(Vol)  # ajoute le volume de la mesure au volume total 
	led13.write(0.5)  # allumer led pendant 0,5 seconde
	UMes = InA0.read()  # tension mesurée V_out [V]
	pRel = (UMes - 0.026) / .08 * 1000  # fonction transfert du capteur de
                                         # pression --> pression en [Pa]
	hCE = pRel / (rho * g)  # hauteur de colone d'eau [m]
	print('Niveau', i+1, '--> UMes =', UMes, 'V ;', 'pRel =', np.round(pRel), 'Pa ;', 'h = ', np.round(hCE, 3), ' m')  # affiche le texte
	#  enregistrement des mesures | n° | Vol | Vout | pRel | hCE | 
	Data[i, 0] = i + 1  # n° de la mesure
	Data[i, 1] = float(VolCumul)  # volume de liquide cumulé dans le contenant en [l] 
	Data[i, 2] = UMes  # tension mesurée
	Data[i, 3] = pRel  # pression relative
	Data[i, 4] = hCE  # hauteur d'eau
	time.sleep(0.25)  # pause
	led13.write(0)  # éteindre led
	time.sleep(0.25)  # pause
	# fin de la boucle for
print('Fin de la prise de mesures')  # affiche le texte
Carte.exit()  # termine l’acquisition de mesures proprement
htheo = 4 * Data[:, 1] / (np.pi * float(D)**2)  # hauteur de liquide théorique [m]
'''-------------------------------------------------------------------------'''

'''--- Figure --------------------------------------------------------------'''
fig, ax1 = plt.subplots(num = 1, figsize=(8, 6))  # attribue un numéro et une taille à la figure 
ax2 = ax1.twinx()  # ajoute un deuxième axe y
ax1.plot(Data[:, 1] * 1e3, Data[:, 3], 'o', mfc = 'steelblue', mec = 'k', ms = 10, label = u'$\Delta p$')  # trace la pression relative en fonction du volume dans le contenant
ax2.plot(Data[:, 1] * 1e3, htheo,  'k', ls = '--', label = u'$h_\mathrm{theo}$')  # trace la hauteur théorique en fonction du volume dans le contenant
ax2.plot(Data[:, 1] * 1e3, Data[:, 4], 'o', mfc = 'steelblue', mec = 'k', ms = 10, label = u'$h_\mathrm{theo}$')  # trace la hauteur mesurée en fonction du volume dans le contenant
ax1.set_xlabel('Volume de liquide [L]')  # nomme l’axe x
ax1.set_ylabel('pression relative  $p_\mathrm{rel}$ [Pa]')  # nomme le premier axe y
ax2.set_ylabel('hauteru d\'eau  $h$ [mCE]')  # nomme le premier axe y
ax1.legend(loc=0)  # affiche la légende de l’axe 1
ax2.legend(loc=3)  # affiche la légende de l’axe 2
plt.savefig('Mesures.pdf')  # enregistre le graphique
'''-------------------------------------------------------------------------'''
